package controller

import (
	"electricity_bill_calc/cache"
	"electricity_bill_calc/exceptions"
	"electricity_bill_calc/model"
	"electricity_bill_calc/repository"
	"electricity_bill_calc/response"
	"electricity_bill_calc/security"
	"electricity_bill_calc/service"
	"net/http"
	"strconv"

	"github.com/gofiber/fiber/v2"
)

func InitializeUserHandlers(router *fiber.App) {
	router.Delete("/login", security.MustAuthenticated, doLogout)
	router.Post("/login", doLogin)
	router.Get("/account", security.ManagementAuthorize, searchUsers)
}

type _LoginForm struct {
	Username string `json:"uname"`
	Password string `json:"upass"`
	Type     int16  `json:"type"`
}

func doLogin(c *fiber.Ctx) error {
	result := response.NewResult(c)
	loginData := new(_LoginForm)
	if err := c.BodyParser(loginData); err != nil {
		return result.Error(http.StatusInternalServerError, "表单解析失败。")
	}
	var (
		session *model.Session
		err     error
	)
	if loginData.Type == model.USER_TYPE_ENT {
		session, err = service.UserService.ProcessEnterpriseUserLogin(loginData.Username, loginData.Password)
	} else {
		session, err = service.UserService.ProcessManagementUserLogin(loginData.Username, loginData.Password)
	}
	if err != nil {
		if authError, ok := err.(*exceptions.AuthenticationError); ok {
			if authError.NeedReset {
				return result.LoginNeedReset()
			}
			return result.Error(int(authError.Code), authError.Message)
		} else {
			return result.Error(http.StatusInternalServerError, err.Error())
		}
	}
	return result.LoginSuccess(session)
}

func doLogout(c *fiber.Ctx) error {
	result := response.NewResult(c)
	session, err := _retreiveSession(c)
	if err != nil {
		return result.Success("用户会话已结束。")
	}
	_, err = cache.ClearSession(session.Token)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	return result.Success("用户已成功登出系统。")
}

func searchUsers(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestPage, err := strconv.Atoi(c.Query("page", "1"))
	if err != nil {
		return result.NotAccept("查询参数[page]格式不正确。")
	}
	requestKeyword := c.Query("keyword")
	requestUserType, err := strconv.Atoi(c.Query("type", "-1"))
	if err != nil {
		return result.NotAccept("查询参数[type]格式不正确。")
	}
	var requestUserStat *bool
	state, err := strconv.ParseBool(c.Query("state"))
	if err != nil {
		requestUserStat = nil
	} else {
		requestUserStat = &state
	}
	users, total, err := repository.UserRepository.FindUser(
		&requestKeyword,
		int16(requestUserType),
		requestUserStat,
		uint(requestPage),
	)
	if err != nil {
		return result.NotFound(err.Error())
	}
	return result.Json(
		http.StatusOK,
		"已取得符合条件的用户集合。",
		response.NewPagedResponse(requestPage, total).ToMap(),
		fiber.Map{"accounts": users},
	)
}
